@@ -1,5 +1,7 @@
 /*
-© 2020 Red Hat, Inc. and others
+SPDX-License-Identifier: Apache-2.0
+
+Copyright Contributors to the Submariner project.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -18,6 +20,7 @@ package controller
 import (
 	"sync"
 
+	"github.com/submariner-io/admiral/pkg/watcher"
 	"k8s.io/client-go/kubernetes"
 
 	"github.com/submariner-io/admiral/pkg/syncer"
@@ -51,13 +54,14 @@ type AgentSpecification struct {
 // and creates an EndpointController in response. The EndpointController will use the app label as filter
 // to listen only for the endpoints event related to ServiceImport created
 type ServiceImportController struct {
-	serviceSyncer       syncer.Interface
-	localClient         dynamic.Interface
-	restMapper          meta.RESTMapper
-	serviceImportSyncer syncer.Interface
-	endpointControllers sync.Map
-	clusterID           string
-	scheme              *runtime.Scheme
+	serviceSyncer        syncer.Interface
+	localClient          dynamic.Interface
+	restMapper           meta.RESTMapper
+	serviceImportSyncer  syncer.Interface
+	endpointControllers  sync.Map
+	clusterID            string
+	scheme               *runtime.Scheme
+	globalIngressIPCache *globalIngressIPCache
 }
 
 // Each EndpointController listens for the endpoints that backs a service and have a ServiceImport
@@ -71,4 +75,14 @@ type EndpointController struct {
 	serviceImportSourceNameSpace string
 	stopCh                       chan struct{}
 	localClient                  dynamic.Interface
+	ingressIPClient              dynamic.NamespaceableResourceInterface
+	isHeadless                   bool
+	globalIngressIPCache         *globalIngressIPCache
+}
+
+type globalIngressIPCache struct {
+	sync.Mutex
+	byService sync.Map
+	byPod     sync.Map
+	watcher   watcher.Interface
 }
